package com.baicizhan.client.business.widget

import android.animation.Animator
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import com.baicizhan.client.business.R
import kotlinx.android.synthetic.main.layout_word_text_view.view.*
import org.jetbrains.annotations.TestOnly
import java.lang.StringBuilder

/**
 * Created by Jacee.
 * Date: 2020.04.13
 */
class WordTextView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = R.attr.word_text_style, defStyleRes: Int = 0) :
    FrameLayout(context, attrs, defStyleAttr, defStyleRes) {

    private var mode: Mode = Mode.NONE

    private var wordDashColor: Int = 0
    private var wordColor: Int = 0
    private var wordStr: String = ""
    private var errorDrawable: Drawable?

    private val dashText: String
        get() = wordStr.split(Regex(" +")).map {
            getDashWord(it)
        }.let {
            StringBuilder().apply {
                it.forEach {
                    append("$it    ")
                }
            }.toString().trimEnd()
        }

    init {
        LayoutInflater.from(context).inflate(R.layout.layout_word_text_view, this)
        minimumHeight = resources.getDimensionPixelSize(R.dimen.word_text_view_min_height)
        val a = context.obtainStyledAttributes(attrs, R.styleable.WordTextView, defStyleAttr, defStyleRes)
        wordDashColor = a.getColor(R.styleable.WordTextView_wordDashColor, Color.BLACK)
        wordColor = a.getColor(R.styleable.WordTextView_wordColor, Color.BLACK)
        wordStr = a.getString(R.styleable.WordTextView_word) ?: ""
        errorDrawable = a.getDrawable(R.styleable.WordTextView_errorSrc)
        a.recycle()
    }

    enum class Mode {
        NONE,
        LOADING,
        ERROR,
        NORMAL
    }

    private fun getDashWord(s: String) = ("_ ".repeat(s.length)).trimEnd()

    override fun onFinishInflate() {
        super.onFinishInflate()
        errorDrawable?.let {
            error_icon.setImageDrawable(it)
        }
        refresh()
    }


    private fun refresh() {
        if (mode < Mode.NORMAL) {
            word_str.text = dashText
            word_str.setTextColor(wordDashColor)
            when (mode) {
                Mode.NONE -> {
                    progress.visibility = View.GONE
                    error_icon.visibility = View.GONE
                }
                Mode.LOADING -> {
                    progress.visibility = View.VISIBLE
                    error_icon.visibility = View.GONE
                }
                Mode.ERROR -> {
                    progress.visibility = View.GONE
                    error_icon.visibility = View.VISIBLE
                    error_icon.alpha = 1f
                    error_icon.animate()
                        .alpha(0f)
                        .setDuration(350)
                        .setStartDelay(1000)
                        .setListener(object : Animator.AnimatorListener {
                            override fun onAnimationRepeat(animation: Animator?) {
                            }

                            override fun onAnimationEnd(animation: Animator?) {
                                error_icon.visibility = View.GONE
                            }

                            override fun onAnimationCancel(animation: Animator?) {
                            }

                            override fun onAnimationStart(animation: Animator?) {
                            }

                        })
                }
                else -> {
                }
            }
        } else {
            word_str.text = wordStr
            word_str.setTextColor(wordColor)
            progress.visibility = View.GONE
            error_icon.visibility = View.GONE
        }
    }

    fun setMode(mode: Mode) {
        this.mode = mode
        refresh()
    }

    fun getMode() = mode

    fun setWord(word: String) {
        wordStr = word
        refresh()
    }

    fun setErrorDrawable(d: Drawable) {
        errorDrawable = d
        error_icon.setImageDrawable(d)
    }

    @TestOnly
    fun testMode() {
        fun next() {
            mode = Mode.values()[(mode.ordinal + 1) % Mode.values().size]
            refresh()
        }
        postDelayed(object : Runnable {
            override fun run() {
                next()
                postDelayed(this, 2000)
            }
        }, 3000)

    }
}

